
// value type

// enum is a type itself
enum NetworkError {
    case NotReachable, Timeout, Unknown
}

enum Direction : String {
    case North = "n"
    case South = "s"
    case East = "e"
    case West = "w"
}

let d: Direction = .East
switch d {
case .North:
    print(d)
case .South, .East:
    print(d.rawValue)
default:
    break
}

let d2 = Direction(rawValue: "w")

extension NetworkError : ErrorType { }

enum Result<T, U where U: ErrorType> {
    case Success(T)
    case Error(U)
}

let result: Result<String, NetworkError> = .Success("Hello, world!")

switch result {
case .Success(let val):
    print(val)
default:
    break
}

enum ArithmeticExpression {
    case Number(Int)
    indirect case Addition(ArithmeticExpression, ArithmeticExpression)
    indirect case Multiplication(ArithmeticExpression, ArithmeticExpression)
    
    func evaluate() -> Int {
        switch self {
        case .Number(let n):
            return n
        case .Addition(let l, let r):
            return l.evaluate() + r.evaluate()
        case .Multiplication(let l, let r):
            return l.evaluate() * r.evaluate()
        }
    }
}

let expr: ArithmeticExpression = .Multiplication(.Number(7), .Addition(.Number(8), .Number(11)))
expr.evaluate()

// all types are non-nullable by default
//let a: [Int] = nil
var b: [Int]? = nil

// compiler is happy to upgrade non-optionals to optionals
var c: [Int]? = [1,2,3]

// optional is an enumeration treated specially by the compiler
let s = Optional<Int>.Some(7)
let n = Optional<Int>.None

enum MyOptional<T> {
    case Some(T)
    case None
}
